/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
============================================================================*/ 
/*============================================================================*/
/** \file yanera_main.c
	\brief This is the main file.

Contains the main function and some utility functions
 */
#include "yanera.h"
#include "yanera_util.h"
#include "yanera_xml_input.h"
#include "yanera_model_build.h"
#include "yanera_profile.h"
#include "yanera_quadrature.h"
#include "yanera_data.h"
#include "yanera_lstsq.h"
#include "yanera_lstsq_constrained.h"
#include "yanera_mdmin.h"
#include "yanera_mdmin_simplex.h"
#include "yanera_siman.h"
#include "yanera_ool.h"
#include "yanera_levmar.h"
#include "yanera_reflectivity.h"
#include "yanera_xml_output.h"
/*============================================================================*/
/** \brief The main function.


The main function accepts one input file only.
Test memory managment with Valgrind.
 */
int main(int argc, char **argv) 
{	
	yanera_container *yanera;
	/*---------------------------------
	 * Program requires a filename of the XML file.
	 */
	if (argc != 2) 
	{
	  yanera_error("No XML inpout file specified.", __FILE__, __LINE__, NULL);
	}
	/*---------------------------------
	 * Allocate a new yanera_container
	 */
	yanera = yanera_initialize(argv[1]);
  /*---------------------------------
   *  Parse the XML file
	 */
	yanera_read_xml(yanera);
	/*---------------------------------
	 * Form the complete models from the XML models in memory
	 */
	yanera_complete_models(yanera);
  /*---------------------------------
	 * Read resolution
	 */
	yanera_read_resolution(yanera);
	/*---------------------------------
	 * Load the data
	 */
	if (yanera->misc.fit_do == YES)	
	{
	  yanera_load_data(yanera);
	  
	  if ((yanera->misc.fit_method == CONJUGATE_FR) || \
	      (yanera->misc.fit_method == CONJUGATE_PR) || \
	      (yanera->misc.fit_method == VECTOR_BFGS) || \
	      (yanera->misc.fit_method == STEEP_DESC))
	  { yanera_mdmin_main(yanera); }
	  if (yanera->misc.fit_method == NMSIMPLEX)
	  { yanera_mdmin_simplex_main(yanera); }
	  if (yanera->misc.fit_method == SIMAN)
	  { yanera_siman_main(yanera); }
#ifdef HAVE_OOL
	  if ((yanera->misc.fit_method == OOL_SPG) || \
	      (yanera->misc.fit_method == OOL_GENCAN))
	  { yanera_ool_main(yanera); }
#endif
	  if ((yanera->misc.fit_method == LMSDR) || \
	      (yanera->misc.fit_method == LMDR))
	  { yanera_lstsq_main(yanera); }
	  if ((yanera->misc.fit_method == CLMSDR) || \
	      (yanera->misc.fit_method == CLMDR))
	  { yanera_lstsq_constrained_main(yanera); }
#ifdef HAVE_LEVMAR
	  if (yanera->misc.fit_method == LEVMAR)
	  { yanera_levmar_main(yanera); }
#endif
	}
  else
  {
    /*---------------------------------
	   * Report results
	   */
	  yanera_report(yanera, -1, NULL);
	}
  
	/*---------------------------------
	 * Free the memory
	 */
  yanera_free(yanera);
  
	return(EXIT_SUCCESS);
}

