/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
=============================================================================*/
/*! \file yanera_quadrature.h
    \brief Breaks down the SLD profile into tiny slabs.
    
This file  containts the necessary function for creating the small slab represnetation of the SLD profile.
 */ 
/*============================================================================*/
/** \brief Trapazoidal rule integration

\param p The full \p yanea_container struct.
\param m The model currently being evaluated.
\param i The slab within the model currently being evaluated.

\returns The vlaue of the trapzoid area of i to i+1

 */
double slabs_trap(yanera_container *p, yanera_model *m, unsigned short i);
/*============================================================================*/
/** \brief Mipoint rule integration

\param p The full \p yanea_container struct.
\param m The model currently being evaluated.
\param i The slab within the model currently being evaluated.

\returns The vlaue of the rectangular area of i to i+1
 */
double slabs_midp(yanera_container *p, yanera_model *m, unsigned short i);
/*============================================================================*/
/** \brief Add a slab.

\param model The model currently being evaluated.
\param j     The new number of the new slab.
\param np    The z position of the new slab.

This function adds a new slab, and renumbers all the slabs at larger z values.
 */
void slabs_insert(yanera_model *model, unsigned short j, double np);
/*============================================================================*/
/** \brief Initializes the slabs for a model

\param model The model currently being evaluated.
\param start The start z position of the profile.
\param end   The end z position of the profile.

This function initalizes a new slab section, starting with one interface and two slabs
 */
void slabs_zero(yanera_model *model, double start, double end);
/*============================================================================*/
/** \brief Do the quadrature.

This function performs the logic of the quadrature. 
 */
void slabs_doit(yanera_model *m, yanera_container *p);
/*============================================================================*/
/** \brief Do the quadrature public function.

This is the function that should be called by fitting routines, etc., when a paramter changes and a new quadrature needs to be performed. It breakes up the profile into little slabs and stores the scattering length density, thickness of each slab so that the reflectivity can be calculated.
 */
void doQuadrature(yanera_model *model, yanera_container *yanera);
/*============================================================================*/
/** \brief Write the SLD profile to file.

If selected in the XML file, this function will write the scattering profile as little slabs to a file. This is useful to examine the correctnes of the quadrature. 
 */
void yanera_write_slabs(yanera_container *yanera);

