/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Parratt; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
============================================================================*/ 
/*============================================================================*/
/** \file yanera_siman.h
\brief Performs the simulated annealing.
  
This file contains the general code for performing simulated annealing, using the GNU scientific library. 
 */
/*============================================================================*/
/** \brief The main function for the simulated annealing fitting routine.

The main function for the simulated annealing. Since this fitting methods does not require any gradiant evaluation, we can simply use \p mdmin_chisq function for evaluation of the \f$\chi^2\f$, and avoid code duplication. 

This method is constrained, simply by not stepping the paramters that are fixed. See the page on \ref fit.
 */
int yanera_siman_main(yanera_container *yanera);
/*============================================================================*/
/*============================================================================*/
/** \brief Calculates the energy cost of \f$\chi^2\f$.

This function is expected to receive a void pointer, whereas \p mdmin_chisq expects a \p yanera_container. So this function casts the void and calls \p mdmin_chisq.
 */
double siman_energy(void *yanera_pointer);
/*============================================================================*/
/** \brief Steps the parameters.

This function uses the random number generator and moves the paramters a maximum amount \p step_size. If the parameters are fixed, or constrained, then appropriate actions are taken.
 */
void siman_step(const gsl_rng *rng, void *yanera_pointer, double step_size);
/*============================================================================*/
/** \brief Calculates the inner product of the parameter array.

There is some confusion whether this function is ever called. The GSL documentation calls for it, so it is included.
 */
double siman_metric(void *yanera_pointer1, void *yanera_pointer2);
/*============================================================================*/
/** \brief Prints to stdout the current state of the parameters.

The output during the simulated annealing is as follows:
- Column: What
- 1: Iteration
- 2: Number of evaluations
- 3: Current temperature
- 4-n: Current state of the parameters
- 5: Current \f$\chi^2\f$
.
 */
void siman_print(void *data);
/*============================================================================*/
/** \brief Copyies the fitting paramters from one \p yanera_container to another.

This function copies the paramters from one \p yanera_container to another.
 */
void siman_copy(void *source_yanera_pointer, void *dest_yanera_pointer);
/*============================================================================*/
/** \brief Creates a new \p yanera_container from another.

Creates a new \p yanera_container from another. It only copies only whatever is important to calculate \f$\chi^2\f$.
 */
void *siman_copy_construct(void *yanera_pointer);
/*============================================================================*/
/** \brief Destroys the working copies of \p yanera_container.

Destroys the working copies of \p yanera_container.
 */
void siman_destroy(void *yanera_pointer);

