/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
==============================================================================*/
/** \file yanera_util.h
    \brief This file contains linked-list and memory utility functions.

This file contains linked-list and memory utility functions.
 */
/*============================================================================*/ 
/** \brief Allocates a new \p yanera_container.

    \returns A newly allocated \p yanera_container.

  Allocated memory for a new \p yanera_container.
 */
yanera_container *yanera_initialize(const char *filename);
/*============================================================================*/
/** \brief Frees the memory

\param yanera The \p yanera_container to free.

This recursively walks through the \p yanera_container, freeing everything that may heve been allocated memory.
 */
void yanera_free(yanera_container *yanera);
/*============================================================================*/
/** \brief General reporting routine for fitting progress updates.


This function wrties to stderr the current status (iteration number, chi sqaured)of the fitting routines. Called with -1 at the iteration number, it assumes the fit is finished and write to file the results, if the proper flags are selected.
 */
void yanera_report(yanera_container *yanera, \
                  int iteration, \
				          void *solver);
/*============================================================================*/
/** \brief Determines the fit parameter errors

This function borrows from the least sqaures to determine the covariance matrix and find the uncertainties of the fitted parameters.
 */
int yanera_find_errors(yanera_container *yanera);
