/*============================================================================
Copyright 2006 Thad Harroun, Kevin Yager

This file is part of "yanera 2.0".

"yanera 2.0" is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

"yanera 2.0" is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with "yanera 2.0"; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
============================================================================*/ 
/*============================================================================*/
/** \file yanera_xml_util.h
    \brief This file contains linked-list and memory utility functions.

This file contains linked-list and memory utility functions, utilized only by the XML related functions.
 */
/*=========================================================================*/
/** \brief Links a \c yanera_layer into the component list.

\param layer The layer to be linked.
\param yanera The \c yanera_container to link the layer into.

  Takes a \c yanera_layer and links it to the end of the list of \c yanera_components found in the \c yanera_container.

 */
void linkComponentIntoContainer(yanera_layer *layer, yanera_container *yanera);
/*=========================================================================*/
/** \brief Adds a paramter to the \c yanera_parameters.

\param d The numerical value of the paramter.
\param i The fix boolean for the paramter.
\param yanera The \c yanera_container to add the parameter to.
\returns The index of the \c yanera_parameters.p array of the newly added paramter.
 */
short newParameter(double d, short i, yanera_container *yanera);
/*=========================================================================*/
/** \brief Adds a variable to the array for a function type layer.

\param c The variable character of the paramter.
\param d The numerical value of the paramter.
\param i The fix boolean for the paramter.
\param layer The \c yanera_layer that is of function type.
\param yanera The \c yanera_container to add the parameter to.
 */
short newFuncParameter(char c, double d, unsigned short i, \
                       yanera_layer *layer, yanera_container *yanera);
/*============================================================================*/
/** \brief Determines the fix flag from a parameter XML property.

\param [in] buf The buffer containing the property of a layer paramter.
\returns A boolean yes or no.

this is a convienience function for repeated use.
 */
unsigned short getYesNoBooleanFromString(char *buf);
/*============================================================================*/
/** \brief Set the buffer for the 'fix' XML property.

\param [in] buf The buffer to hold the property of a layer paramter.
\param [in] i1 The boolean flag.

IMPORTANT: The buffer memory must be prealocated and of at least size 4.
 */
void setYesNoBooleanFromString(char *buf, short i1);
/*============================================================================*/
/** \brief Allocates a new \p yanera_layer

\returns A newly allocated \p yanera_layer
 */
yanera_layer *newLayer(short type);
/*============================================================================*/
/** \brief Makes a copy of a \p yanera_layer

\returns A newly allocated \p yanera_layer.
 */
yanera_layer *copyLayer(yanera_layer *source);
/*============================================================================*/
/** \brief Links a \p yanera_layer into a \p yanera_model

\param layer The \p yanera_layer to link.
\param model The \p yanera_model with the linked list of \p yanera_model.
 */
void linkLayerIntoModel(yanera_layer *layer, yanera_model *model);
/*============================================================================*/
/** \brief Function to catch the strucutred errors of XML parsing.

\param ctx Not sure what this should be, treating as xmlValidCtxt
\param error The xmlError structure, containing error message and location.
 */
void	xmlErrorFunc(void *ctx, xmlErrorPtr error);

